package com.game.business.controller;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.game.business.domain.AppUser;
import com.game.business.domain.FinTranRecord;
import com.game.business.dto.AppUserDividendListDto;
import com.game.business.dto.AppUserDividendSendDto;
import com.game.business.service.IAppUserService;
import com.game.business.service.IFinTranRecordService;
import com.game.common.constant.AppSceneType;
import com.game.common.constant.finance.FinTranAddedInfo;
import com.game.common.constant.finance.FinTranType1;
import com.game.common.constant.finance.FinTranType3;
import com.game.common.constant.finance.TranCurrencyType;
import com.game.common.core.domain.HttpRet;
import com.game.common.core.domain.HttpRetPageArr;
import com.game.common.core.domain.R;
import com.game.common.core.redis.RedisCache;
import com.game.common.utils.MoneyUtils;
import com.game.common.utils.SecurityUtils;
import com.game.common.utils.StringUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.game.common.annotation.Log;
import com.game.common.core.controller.BaseController;
import com.game.common.core.domain.AjaxResult;
import com.game.common.enums.BusinessType;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.game.business.domain.AppUserCountDividend;
import com.game.business.service.IAppUserCountDividendService;
import com.game.common.utils.poi.ExcelUtil;
import com.game.common.core.page.TableDataInfo;

/**
 * 用户代理分红Controller
 * 
 * @author game
 * @date 2024-06-30
 */
@RestController
@RequestMapping("/business/dividend")
@Api(value = "AppUserCountDividendController", description = "用户代理表接口", tags = {"用户代理表接口"})
public class AppUserCountDividendController extends BaseController
{
    @Autowired
    private IAppUserCountDividendService appUserCountDividendService;

    @Autowired
    private IAppUserService appUserService;

    @Autowired
    private IFinTranRecordService finTranRecordService;

    @Autowired
    private RedisCache redisCache;

    /**
     * 查询用户代理分红列表
     */
    @GetMapping("/list")
    @ApiOperation(value = "查询周分红列表", notes = "获取周分红列表")
    public HttpRetPageArr<AppUserCountDividend> list(AppUserDividendListDto dto)
    {
        startPage();
        List<AppUserCountDividend> list = appUserCountDividendService.selectUserDividendList(SecurityUtils.getUserId());
        return HttpRetPageArr.success("查询成功", getDataTable(list));
    }

    /**
     * 发放分红
     */
    @PostMapping("/send")
    @ApiOperation(value = "发放分红", notes = "发放分红")
//    @Transactional

    public HttpRet<Boolean> send(AppUserDividendSendDto dto)
    {
        String userId = dto.getUserId();
        String dateNo = dto.getDateNum();
        Long curUserId = SecurityUtils.getUserId();
        if(null == userId && StringUtils.isBlank(dateNo)){
            return HttpRet.fail("发放失败，参数为空");
        }
        LambdaQueryWrapper<AppUserCountDividend> queryWrapper = new LambdaQueryWrapper<>();
        if(StringUtils.isNotBlank(dateNo)){
            queryWrapper.eq(AppUserCountDividend::getDateNo,dateNo);
        }
        if(null != userId){
            queryWrapper.eq(AppUserCountDividend::getUserId,userId);
        }
        queryWrapper.eq(AppUserCountDividend::getStatus,0);
        List<AppUserCountDividend> dividendList = appUserCountDividendService.list(queryWrapper);
        if(null == dividendList || dividendList.size() < 1){
            return HttpRet.fail("发放失败，不存在或已发放");
        }
        AppUser curUser = appUserService.selectAppUserByUserid(curUserId);
        if(curUser.getDiamondCoin() < dividendList.stream().mapToDouble(e->e.getDiamondCoin()).sum()){
            return HttpRet.fail("发放失败，余额不足");
        }
        if(curUser.getCoin() < dividendList.stream().mapToDouble(e->e.getCoin()).sum()){
            return HttpRet.fail("发放失败，金币不足");
        }
        //开始发放
        List<Long> ids = dividendList.stream().map(AppUserCountDividend::getUserId).collect(Collectors.toList());
        List<AppUser> userList = appUserService.selectListByIds(ids);
        if(null != userList && userList.size()>0) {
            Map<String, AppUser> userMap = userList.stream().collect(Collectors.toMap(e -> String.valueOf(e.getUserid()), Function.identity(), (k1, k2) -> k2));
            dividendList.forEach(e->{
                List<FinTranRecord> tranRecordList = new ArrayList<>();
                AppUser appUser = userMap.get(String.valueOf(e.getUserId()));
                AppUser updateAppUser = new AppUser();
                AppUser curUpdateAppUser = new AppUser();//当前用户
                updateAppUser.setUserid(appUser.getUserid());
                if(null != e.getDiamondCoin() && e.getDiamondCoin()>0){
                    //扣减发放用户
                    curUser.setDiamondCoinCash(curUser.getDiamondCoinCash() - MoneyUtils.consumeCash(curUser.getDiamondCoin(),curUser.getDiamondCoinCash(),e.getDiamondCoin()));
                    curUser.setDiamondCoin(curUser.getDiamondCoin() - e.getDiamondCoin());
                    curUser.setDiamondCoinTotal(curUser.getDiamondCoinTotal() - e.getDiamondCoin());
                    curUpdateAppUser.setUserid(curUser.getUserid());
                    curUpdateAppUser.setDiamondCoinCash(curUser.getDiamondCoinCash());
                    curUpdateAppUser.setDiamondCoin(curUser.getDiamondCoin());
                    curUpdateAppUser.setDiamondCoinTotal(curUser.getDiamondCoinTotal());
                    //发放用户扣减流水
                    FinTranAddedInfo curAddedInfo = FinTranAddedInfo.createTranInfo(appUser.getUserid(), 0, 0, AppSceneType.Scene_None, "");
                    FinTranRecord curTran = FinTranRecord.initFinTranRecordSomeParams(curAddedInfo, FinTranType3.AGENT_DIVIDED, FinTranType1.U_Outcome_Balance, curUser);
                    curTran.setDiamondCoinChange(e.getDiamondCoin() * -1);
                    curTran.setAfterDiamondCoin(curUser.getDiamondCoin());
                    curTran.setCurrencyType(TranCurrencyType.Balance.getType());
                    curTran.setRemarks(e.getDateNo() + " 发放团队分红奖励");
                    curTran.setFromUid(curUserId);
                    tranRecordList.add(curTran);

                    //受益用户
                    updateAppUser.setDiamondCoin(appUser.getDiamondCoin() + e.getDiamondCoin());
                    updateAppUser.setDiamondCoinTotal(appUser.getDiamondCoinTotal() + e.getDiamondCoin());
                    updateAppUser.setDiamondCoinCash(appUser.getDiamondCoinCash() + e.getDiamondCoin());
                    //流水 余额
                    FinTranAddedInfo addedInfo = FinTranAddedInfo.createTranInfo(appUser.getUserid(), 0, 0, AppSceneType.Scene_None, "");
                    FinTranRecord tran = FinTranRecord.initFinTranRecordSomeParams(addedInfo, FinTranType3.AGENT_DIVIDED, FinTranType1.Agent_Income_Diamond_Coin, appUser);
                    tran.setDiamondCoinChange(e.getDiamondCoin());
                    tran.setAfterDiamondCoin(appUser.getDiamondCoin() + e.getDiamondCoin());
                    tran.setCurrencyType(TranCurrencyType.Balance.getType());
                    tran.setRemarks(e.getDateNo() + " 团队周分红奖励");
                    tran.setFromUid(curUserId);
                    tranRecordList.add(tran);
                }
                if(null != e.getCoin() && e.getCoin()>0){
                    //扣减发放用户
                    curUser.setCoinCash(curUser.getCoinCash() - MoneyUtils.consumeCash(curUser.getCoin(),curUser.getCoinCash(),e.getCoin()));
                    curUser.setCoin(curUser.getCoin() - e.getCoin());
                    curUpdateAppUser.setUserid(curUser.getUserid());
                    curUpdateAppUser.setCoinCash(curUser.getCoinCash());
                    curUpdateAppUser.setCoin(curUser.getCoin());
                    //发放用户扣减流水
                    FinTranAddedInfo curAddedInfo = FinTranAddedInfo.createTranInfo(appUser.getUserid(), 0, 0, AppSceneType.Scene_None, "");
                    FinTranRecord curTran = FinTranRecord.initFinTranRecordSomeParams(curAddedInfo, FinTranType3.AGENT_DIVIDED, FinTranType1.U_Outcome_Coin, curUser);
                    curTran.setDiamondCoinChange(e.getDiamondCoin() * -1);
                    curTran.setAfterCoin(curUser.getCoin());
                    curTran.setCurrencyType(TranCurrencyType.Coin.getType());
                    curTran.setRemarks(e.getDateNo() + " 发放团队分红奖励");
                    curTran.setFromUid(curUserId);
                    tranRecordList.add(curTran);

                    updateAppUser.setCoin(appUser.getDiamondCoin() + e.getDiamondCoin());
                    //流水 余额
                    FinTranAddedInfo addedInfo = FinTranAddedInfo.createTranInfo(appUser.getUserid(), 0, 0, AppSceneType.Scene_None, "");
                    FinTranRecord tran = FinTranRecord.initFinTranRecordSomeParams(addedInfo, FinTranType3.AGENT_DIVIDED, FinTranType1.Agent_Income_Coin, appUser);
                    tran.setDiamondCoinChange(e.getDiamondCoin());
                    tran.setAfterDiamondCoin(appUser.getDiamondCoin() + e.getDiamondCoin());
                    tran.setCurrencyType(TranCurrencyType.Coin.getType());
                    tran.setRemarks(e.getDateNo() + " 团队周分红奖励");
                    tran.setFromUid(curUserId);
                    tranRecordList.add(tran);
                }
                appUserService.updateAppUser(updateAppUser);
                appUserService.updateAppUser(curUpdateAppUser);
                //清除用户缓存
                redisCache.deleteObject("U:UserInfo:".concat(String.valueOf(appUser.getUserid())));
                if(tranRecordList.size() > 0){
                    tranRecordList.forEach(t->{
                        finTranRecordService.insertFinTranRecord(t);
                    });
                }
                AppUserCountDividend userCountDividend = new AppUserCountDividend();
                userCountDividend.setId(e.getId());
                userCountDividend.setStatus(1);
                appUserCountDividendService.updateAppUserCountDividend(userCountDividend);
            });
        }
        return HttpRet.success("发放成功",true);
        /*
        return appUserCountDividendService.send(dto.getUserId(),dto.getDateNum());*/
    }

}
