package com.game.business.service.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Date;
import java.util.function.Function;
import java.util.stream.Collectors;

import com.game.business.domain.*;
import com.game.business.service.*;
import com.game.common.constant.AppSceneType;
import com.game.common.constant.CacheConstants;
import com.game.common.constant.finance.FinTranAddedInfo;
import com.game.common.constant.finance.FinTranType1;
import com.game.common.constant.finance.FinTranType3;
import com.game.common.constant.finance.TranCurrencyType;
import com.game.common.core.domain.HttpRet;
import com.game.common.core.redis.RedisCache;
import com.game.common.utils.DateUtils;
import com.game.common.utils.StringUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import com.game.business.mapper.AppUserCountDividendMapper;
import com.game.common.annotation.DataSource;
import com.game.common.enums.DataSourceType;
import org.springframework.transaction.annotation.Transactional;

/**
 * 用户代理分红Service业务层处理
 *
 * @author game
 * @date 2024-06-30
 */
@Service
@Log4j2
public class AppUserCountDividendServiceImpl extends ServiceImpl<AppUserCountDividendMapper, AppUserCountDividend> implements IAppUserCountDividendService {
    @Autowired
    private AppUserCountDividendMapper appUserCountDividendMapper;
    @Autowired
    private IAppUserService appUserService;
    @Autowired
    private IFinTranRecordService finTranRecordService;
    @Autowired
    private IAppUserCountService appUserCountService;
    @Autowired
    private RedisCache redisCache;
    @Autowired
    private IAppUserAgentService appUserAgentService;

    /**
     * 查询用户代理分红
     *
     * @param id 用户代理分红主键
     * @return 用户代理分红
     */
    @Override
    public AppUserCountDividend selectAppUserCountDividendById(Long id) {
        return appUserCountDividendMapper.selectAppUserCountDividendById(id);
    }

    /**
     * 查询用户代理分红列表
     *
     * @param appUserCountDividend 用户代理分红
     * @return 用户代理分红
     */
    @Override
    public List<AppUserCountDividend> selectAppUserCountDividendList(AppUserCountDividend appUserCountDividend) {
        return appUserCountDividendMapper.selectAppUserCountDividendList(appUserCountDividend);
    }

    /**
     * 新增用户代理分红
     *
     * @param appUserCountDividend 用户代理分红
     * @return 结果
     */
    @Override
    @Async
    public void insertAppUserCountDividend(AppUserCountDividend appUserCountDividend) {
                appUserCountDividend.setCreateTime(DateUtils.getNowDate());
          LambdaQueryWrapper<AppUserCountDividend> queryWrapper = new LambdaQueryWrapper<>();
          queryWrapper.eq(AppUserCountDividend::getUserId, appUserCountDividend.getUserId());
          queryWrapper.eq(AppUserCountDividend::getDateNo, appUserCountDividend.getDateNo());
          AppUserCountDividend userCountDividend = appUserCountDividendMapper.selectOne(queryWrapper);
          if (userCountDividend == null) {
              appUserCountDividend.setCreateTime(DateUtils.getNowDate());
              appUserCountDividendMapper.insert(appUserCountDividend);
          }else if(userCountDividend.getStatus().intValue() == 0){
              appUserCountDividend.setUpdateTime(DateUtils.getNowDate());
              appUserCountDividend.setId(userCountDividend.getId());
              appUserCountDividendMapper.updateById(appUserCountDividend);
          }else{
              log.warn("当前用户已发放分红，无法更新未发放");
          }
    }

    /**
     * 修改用户代理分红
     *
     * @param appUserCountDividend 用户代理分红
     * @return 结果
     */
    @Override
    public int updateAppUserCountDividend(AppUserCountDividend appUserCountDividend) {
                appUserCountDividend.setUpdateTime(DateUtils.getNowDate());
        return appUserCountDividendMapper.updateAppUserCountDividend(appUserCountDividend);
    }

    /**
     * 批量删除用户代理分红
     *
     * @param ids 需要删除的用户代理分红主键
     * @return 结果
     */
    @Override
    public int deleteAppUserCountDividendByIds(Long[] ids) {
        return appUserCountDividendMapper.deleteAppUserCountDividendByIds(ids);
    }

    /**
     * 删除用户代理分红信息
     *
     * @param id 用户代理分红主键
     * @return 结果
     */
    @Override
    public int deleteAppUserCountDividendById(Long id) {
        return appUserCountDividendMapper.deleteAppUserCountDividendById(id);
    }

    @Override
    public List<AppUserCountDividend> selectUserDividendList(Long userId) {
        List<AppUserCountDividend> dividendList = appUserCountDividendMapper.selectUserDividendList(userId);
        if(null != dividendList && dividendList.size()>0){
            List<Long> ids = dividendList.stream().map(AppUserCountDividend::getUserId).collect(Collectors.toList());
            List<AppUser> userList = appUserService.selectListByIds(ids);
            if(null != userList && userList.size()>0){
                Map<String,AppUser> userMap = userList.stream().collect(Collectors.toMap(e->String.valueOf(e.getUserid()), Function.identity(),(k1,k2)->k2));
                dividendList.forEach(e->{
                    AppUser appUser = userMap.get(String.valueOf(e.getUserId()));
                    if(null != appUser){
                        e.setNickName(appUser.getNickname());
                    }
                });
            }
        }
        return dividendList;
    }

    @Override
    @Transactional
    public HttpRet<Boolean> send(String userId, String dateNo) {
        if(null == userId && StringUtils.isBlank(dateNo)){
            return HttpRet.fail("发放失败，参数为空");
        }
        LambdaQueryWrapper<AppUserCountDividend> queryWrapper = new LambdaQueryWrapper<>();
        if(StringUtils.isNotBlank(dateNo)){
            queryWrapper.eq(AppUserCountDividend::getDateNo,dateNo);
        }
        if(null != userId){
            queryWrapper.eq(AppUserCountDividend::getUserId,userId);
        }
        queryWrapper.eq(AppUserCountDividend::getStatus,0);
        List<AppUserCountDividend> dividendList = appUserCountDividendMapper.selectList(queryWrapper);
        if(null == dividendList || dividendList.size() < 1){
            return HttpRet.fail("发放失败，不存在或已发放");
        }
        //开始发放
        List<Long> ids = dividendList.stream().map(AppUserCountDividend::getUserId).collect(Collectors.toList());
        List<AppUser> userList = appUserService.selectListByIds(ids);
        if(null != userList && userList.size()>0) {
            Map<String, AppUser> userMap = userList.stream().collect(Collectors.toMap(e -> String.valueOf(e.getUserid()), Function.identity(), (k1, k2) -> k2));
            dividendList.forEach(e->{
                List<FinTranRecord> tranRecordList = new ArrayList<>();
                AppUser appUser = userMap.get(String.valueOf(e.getUserId()));
                AppUser updateAppUser = new AppUser();
                updateAppUser.setUserid(appUser.getUserid());
                if(null != e.getDiamondCoin() && e.getDiamondCoin()>0){
                    updateAppUser.setDiamondCoin(appUser.getDiamondCoin() + e.getDiamondCoin());
                    updateAppUser.setDiamondCoinTotal(appUser.getDiamondCoinTotal() + e.getDiamondCoin());
                    updateAppUser.setDiamondCoinCash(updateAppUser.getDiamondCoinCash() + e.getDiamondCoin());
                    //流水 余额
                    FinTranAddedInfo addedInfo = FinTranAddedInfo.createTranInfo(appUser.getUserid(), 0, 0, AppSceneType.Scene_None, "");
                    FinTranRecord tran = FinTranRecord.initFinTranRecordSomeParams(addedInfo, FinTranType3.AGENT_DIVIDED, FinTranType1.Agent_Income_Diamond_Coin, appUser);
                    tran.setDiamondCoinChange(e.getDiamondCoin());
                    tran.setAfterDiamondCoin(appUser.getDiamondCoin() + e.getDiamondCoin());
                    tran.setCurrencyType(TranCurrencyType.Balance.getType());
                    tranRecordList.add(tran);
                }
                if(null != e.getCoin() && e.getCoin()>0){
                    updateAppUser.setCoin(appUser.getDiamondCoin() + e.getDiamondCoin());
                    //流水 余额
                    FinTranAddedInfo addedInfo = FinTranAddedInfo.createTranInfo(appUser.getUserid(), 0, 0, AppSceneType.Scene_None, "");
                    FinTranRecord tran = FinTranRecord.initFinTranRecordSomeParams(addedInfo, FinTranType3.AGENT_DIVIDED, FinTranType1.Agent_Income_Coin, appUser);
                    tran.setDiamondCoinChange(e.getDiamondCoin());
                    tran.setAfterDiamondCoin(appUser.getDiamondCoin() + e.getDiamondCoin());
                    tran.setCurrencyType(TranCurrencyType.Coin.getType());
                    tranRecordList.add(tran);
                }
                appUserService.updateAppUser(updateAppUser);
                if(tranRecordList.size() > 0){
                    tranRecordList.forEach(t->{
                        finTranRecordService.insertFinTranRecord(t);
                    });
                }
                AppUserCountDividend userCountDividend = new AppUserCountDividend();
                userCountDividend.setId(e.getId());
                userCountDividend.setStatus(1);
                this.updateAppUserCountDividend(userCountDividend);
            });
        }
        return HttpRet.success("发放成功",true);
    }

    @Override
    public void reloadCache(String dateTime, List<Long> userIds) {
        Date uDate = new Date();
        if (StringUtils.isNotBlank(dateTime)) {
            uDate = DateUtil.parse(dateTime, "yyyy-MM-dd");
        }
        String beginTime = DateUtil.format(DateUtil.beginOfWeek(uDate), "yyyy-MM-dd");
        String endTime = DateUtil.format(DateUtil.endOfWeek(uDate), "yyyy-MM-dd");
        if (null == userIds) {
            List<AppUser> list = appUserService.selectTeamUserList();
            if (null != list && list.size() > 0) {
                list.forEach(appUser -> {
                    double money = calculation(appUser.getUserid(), beginTime, endTime);
                    redisCache.setCacheMapValue(CacheConstants.USER_AGENT_TEAM_PHASE, String.valueOf(appUser.getUserid()), money);
                });
            }
        } else {
            userIds.forEach(e->{
                AppUserAgent appUserAgent = appUserAgentService.selectInfo(e);
                if (null != appUserAgent) {
                    Long topUserId = appUserAgent.getTopId()<1?appUserAgent.getUserId():appUserAgent.getTopId();
                    double money = calculation(topUserId, beginTime, endTime);
                    redisCache.setCacheMapValue(CacheConstants.USER_AGENT_TEAM_PHASE, String.valueOf(topUserId), money);
                }else{
                    log.info("用户非代理停止计算周盈亏缓存");
                }
            });
        }
    }

    private double calculation(Long userId,String beginTime,String endTime){
        double money = 0;
        List<AppUserCount> appUserCountList = appUserCountService.getAppUserCountNew(userId,beginTime,endTime);
        if (null != appUserCountList && appUserCountList.size() > 0) {
            //游戏输钱
            double loseAmount = appUserCountList.stream().mapToDouble(e -> e.getGameLoseAmount()).sum();
            //游戏赢钱
            double winAmount = appUserCountList.stream().mapToDouble(e -> e.getGameWinAmount()).sum();
            //提现手续费
            double withdrawalCommission = appUserCountList.stream().mapToDouble(e -> e.getWithdrawalCommission()).sum();
            //游戏佣金
            double gameCommission = appUserCountList.stream().mapToDouble(e -> e.getGameCommission()).sum();
            //直播佣金
            double liveCommission = appUserCountList.stream().mapToDouble(e -> e.getLiveCommission()).sum();
            //充值赠送
            double rechargeGive = appUserCountList.stream().mapToDouble(e -> e.getRechargeGive()).sum();

            //计算分红金额 （输-赢）
//                        double money = (loseAmount - winAmount) - withdrawalCommission - gameCommission - liveCommission;
            money = (loseAmount - winAmount)  - gameCommission - rechargeGive;

        }
        return money;
    }


}